function T = MSTJbi(X,part)
%MSTJ applica l'algoritmo di Jarnik Prim per trovare il mst
%dato un grafo euclideo bipartito

[n,~]=size(X);
%tabella che nelle righe contiene gli archi aggiunti nel grafo nella forma
%(predecessore, nodo, peso)
grafo=zeros(n-1,3);

%coda di priorit in forma di heap binario che contiene i nodi
%non ancora nel MST e la distanza minima dall'albero gi costruito
PQ=inf*ones(n-1,2);
PQ(:,1)=(2:n)';


%posizioni dei nodi all'interno dell'heap
pos=zeros(n,1);
pos(2:n)=1:n-1;

%calcola i pesi degli archi connessi al nodo 1 e li inserisce nel grafo
connodes=(2:n)'.*(part(2:n)~=part(1));
connodes=nonzeros(connodes);
s=sum(part(2:n)~=part(1));
vedges=vecnorm((X(nonzeros(connodes),:)-ones(s,1)*X(1,:)),2,2);
for k=1:s
    [PQ,pos]=decreasekey(PQ,n-1,pos,pos(connodes(k)),vedges(k));
end

%vettore dei predecessori
pred= ones(n,1);

%numero dei nodi ancora da inserire
n=n-1;

while ~isempty(PQ)
    
    %estrae il nodo pi vicino e lo aggiunge al MST con il relativo arco
    v=PQ(1,:);
    grafo(n,:)=[v(1),pred(v(1)),v(2)];
    [PQ,n,pos] = dequeue(PQ,n,pos);
   
    
    %vettore delle distanze dall'ultimo nodo agguinto dei nodi
    %non ancora nel MST    
    connodes=PQ(:,1).*(part(PQ(:,1))~=part(v(1)));
    connodes=nonzeros(connodes);
    s=sum(part(PQ(:,1))~=part(v(1)));
    vedges=vecnorm((X((connodes),:)-ones(s,1)*X(v(1),:)),2,2) ;
    
    %estrae solo le distanze che sono minori rispetto al MST con n-1 nodi
    vedges=vedges.*(PQ(pos(connodes),2)>vedges);
    
    %aggiorna nella coda di priorit le distanze dei nodi, mantendendo la
    %struttura di heap, e il vettore dei predecessori
    for j=1:s
        if vedges(j)~=0
            pred(connodes(j))=v(1);
            [PQ,pos]=decreasekey(PQ,n,pos,pos(connodes(j)),vedges(j));
        end
    end
  
end

   Tab=table(grafo(:,1:2), grafo(:,3));
Tab.Properties.VariableNames={'EndNodes','Weight'};
T=graph(Tab); 
    
end